# Generated by Django 2.2.11 on 2020-03-27 10:52

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bridger', '0003_auto_20200218_1329'),
    ]

    operations = [
        migrations.AddField(
            model_name='notification',
            name='endpoint',
            field=models.URLField(null=True),
        ),
        migrations.AlterField(
            model_name='notification',
            name='timestamp_created',
            field=models.DateTimeField(auto_now_add=True, verbose_name='Created'),
        ),
        migrations.AlterField(
            model_name='notification',
            name='timestamp_mailed',
            field=models.DateTimeField(blank=True, null=True, verbose_name='Mailed'),
        ),
        migrations.AlterField(
            model_name='notification',
            name='timestamp_read',
            field=models.DateTimeField(blank=True, null=True, verbose_name='Read'),
        ),
        migrations.AlterField(
            model_name='notification',
            name='timestamp_received',
            field=models.DateTimeField(blank=True, null=True, verbose_name='Received'),
        ),
    ]



from .buttons import ButtonMetadata, ButtonMetadataMixin
from .create_buttons import CreateButtonMetadata, CreateButtonMetadataMixin
from .custom_buttons import CustomButtonMetadata, CustomButtonMetadataMixin
from .custom_instance_buttons import (
    CustomInstanceButtonMetadata,
    CustomInstanceButtonMetadataMixin,
)
from .display import (
    InstanceDisplayMetadata,
    InstanceDisplayMetadataMixin,
    ListDisplayMetadata,
    ListDisplayMetadataMixin,
)
from .endpoints import EndpointMetadata, EndpointMetadataMixin
from .fields import FieldsMetadata, FieldsMetadataMixin
from .filter_fields import FilterFieldsMetadata, FilterFieldsMetadataMixin
from .identifier import IdentifierMetadata, IdentifierMetadataMixin
from .ordering_fields import OrderingFieldsMetadata, OrderingFieldsMetadataMixin
from .pagination import PaginationMetadata, PaginationMetadataMixin
from .pk import PKMetadata, PKMetadataMixin
from .search_fields import SearchFieldsMetadata, SearchFieldsMetadataMixin
from .titles import TitleMetadata, TitleMetadataMixin
from .widget_type import WidgetTypeMetadata, WidgetTypeMetadataMixin

from .preview import PreviewMetadata, PreviewMetadataMixin
from .docs import DocsMetadata, DocsMetadataMixin


from typing import Dict

from django.template import Context, Template
from django.utils.safestring import SafeString

from bridger.settings import bridger_settings


def load_registered_templatetags() -> str:
    """Assembles the templatetags that are registered to bridger in the django load format"""
    if templatetags := bridger_settings.MARKDOWN_TEMPLATE_TAGS or "":
        templatetags = f"{{% load {' '.join(templatetags)} %}}"

    return templatetags


def compile_template_for_templatetag(templatetag: str) -> Template:
    """Compiles a template with the registered templatetags to resolve a single templatetag"""
    return Template(load_registered_templatetags() + templatetag)


def render_template_for_templatetag(templatetag: str, **context: Dict) -> SafeString:
    """Renders a template for a single templatetag"""
    template = compile_template_for_templatetag(templatetag)
    return template.render(Context(context))



# Generated by Django 2.2.9 on 2020-01-02 02:04

import uuid

import django.contrib.postgres.fields.jsonb
import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="FrontendUserConfiguration",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "config",
                    django.contrib.postgres.fields.jsonb.JSONField(
                        blank=True, default=dict, null=True
                    ),
                ),
                (
                    "parent_configuration",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="child_configurations",
                        to="bridger.FrontendUserConfiguration",
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="configurations",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "verbose_name": "Frontend User Settings",
                "verbose_name_plural": "Frontend User Settings",
            },
        ),
    ]



from django.http import HttpResponse
from django.core.exceptions import ValidationError

from django.template import Template, Context, exceptions

from rest_framework.permissions import IsAuthenticated
from rest_framework.request import Request
from rest_framework.response import Response
from rest_framework.reverse import reverse
from rest_framework.views import APIView
from rest_framework import status

from markdown_blockdiag.utils import draw_blockdiag, DIAG_MODULES
from blockdiag.parser import ParseException

from bridger.settings import bridger_settings

from .template import render_template_for_templatetag
from .models import Asset


class BlockDiag(APIView):
    permission_classes = [IsAuthenticated]

    def post(self, request: Request) -> Response:
        try:
            blockdiag = request.data["markdown"]
            svg = draw_blockdiag(blockdiag, output_fmt="svg", font_antialias=True)
        except (ParseException, KeyError):
            return Response(status=status.HTTP_400_BAD_REQUEST)
        else:
            return Response(svg)


class TemplateTagView(APIView):
    permission_classes = []

    def post(self, request: Request) -> Response:
        if templatetag := request.data.get("templatetag"):
            try:
                return Response(
                    render_template_for_templatetag(templatetag, request=request)
                )
            except exceptions.TemplateSyntaxError:
                return Response("malformatted templatetag", status=400)
            except Exception as e:
                return Response("not handled exception", status=400)
        return Response("templatetag missing", status=400)


class AssetCreateView(APIView):
    permission_classes = []

    def post(self, request: Request) -> Response:
        try:
            asset = Asset.objects.create(file=request.data["file"])
            return Response(
                reverse("bridger:asset-retrieve", args=[asset.id], request=request)
            )
        except KeyError:
            return Response("file missing", status=400)


class AssetRetrieveView(APIView):
    permission_classes = []

    def get(self, request: Request, uuid: str) -> Response:
        try:
            asset = Asset.objects.get(id=uuid)
            response = HttpResponse(asset.file, content_type=asset.content_type)
            response[
                "Content-Disposition"
            ] = f"attachment; filename={asset.file_url_name}"
            return response
        except Asset.DoesNotExist:
            return Response(status=404)
        except ValidationError as e:
            return Response(*e, status=400)



# Generated by Django 3.0.6 on 2020-05-14 15:11

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bridger', '0007_asset'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='Asset',
            new_name='Asset111',
        ),
    ]



from rest_framework import filters
from rest_framework.request import Request

from bridger.metadata.mixins import BridgerMetadataMixin


class OrderingFieldsMetadata(BridgerMetadataMixin):
    key = "ordering_fields"
    method_name = "_get_ordering_fields"


class OrderingFieldsMetadataMixin:
    def get_ordering_fields(self, request: Request):
        if filters.OrderingFilter in getattr(self, "filter_backends", []):
            for ordering_field in self.ordering_fields:
                if "__" in ordering_field:
                    yield ordering_field.split("__")[0], ordering_field
                else:
                    yield ordering_field, ordering_field

    def _get_ordering_fields(self, request: Request):
        return dict(self.get_ordering_fields(request=request))



import logging
import uuid

import django_filters
from django.conf import settings
from django.contrib.auth import get_user_model
from django.contrib.postgres.fields import JSONField
from django.db import models
from django_filters.rest_framework import DjangoFilterBackend
from rest_framework.filters import OrderingFilter

from .filters import BooleanFilter, CharFilter, ModelChoiceFilter
from .serializers import ModelSerializer, PrimaryKeyCharField
from .settings import bridger_settings
from .viewsets import InfiniteDataModelView

logger = logging.getLogger(__name__)


class FrontendUserConfiguration(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    user = models.ForeignKey(
        to=get_user_model(), related_name="configurations", on_delete=models.CASCADE
    )

    parent_configuration = models.ForeignKey(
        to="self",
        related_name="child_configurations",
        null=True,
        blank=True,
        on_delete=models.CASCADE,
    )
    config = JSONField(default=dict, null=True, blank=True)

    def __str__(self):
        return f"{self.user} {self.id}"

    class Meta:
        verbose_name = "Frontend User Settings"
        verbose_name_plural = "Frontend User Settings"


class FrontendUserConfigurationModelSerializer(ModelSerializer):
    id = PrimaryKeyCharField()

    def validate(self, data):
        data["user"] = self.context["request"].user
        return super().validate(data)

    class Meta:
        model = FrontendUserConfiguration
        fields = ["id", "parent_configuration", "config"]


class FrontendUserConfigurationFilterSet(django_filters.rest_framework.FilterSet):
    is_root = BooleanFilter(label="Is Root", method="get_is_root")
    parent_configuration = ModelChoiceFilter(
        label="Parent Configuration", queryset=FrontendUserConfiguration.objects.all()
    )
    config = CharFilter(label="Config", method="filter_config")

    def get_is_root(self, queryset, name, value):
        return queryset.filter(parent_configuration__isnull=value)

    def filter_config(self, queryset, name, value):
        try:
            key, value = value.split(":")
            if value in ["true", "false"]:
                value = True if value == "true" else False
            return queryset.filter(**{f"config__{key}": value})
        except ValueError:
            return queryset

    class Meta:
        model = FrontendUserConfiguration
        fields = ["is_root", "config"]


class FrontendUserConfigurationModelViewSet(InfiniteDataModelView):
    INSTANCE_BUTTONS = LIST_BUTTONS = []
    serializer_class = FrontendUserConfigurationModelSerializer
    filter_backends = [OrderingFilter, DjangoFilterBackend]
    filter_class = FrontendUserConfigurationFilterSet

    ordering_fields = ordering = [
        "id",
        *bridger_settings.DEFAULT_FRONTEND_USER_CONFIGURATION_ORDER,
    ]

    def get_queryset(self):
        return FrontendUserConfiguration.objects.filter(user=self.request.user)




from django.db.models import QuerySet
from rest_framework.request import Request
from rest_framework.response import Response
from rest_framework.viewsets import ViewSet, ViewSetMixin

from bridger.enums import Button, WidgetType
from bridger.fsm.mixins import FSMViewSetMixin
from bridger.metadata.views import MetadataMixin
from bridger.pagination import CursorPagination

from .generics import GenericAPIView
from .mixins import (
    CreateModelMixin,
    DestroyModelMixin,
    DestroyMultipleModelMixin,
    FilterMixin,
    ListModelMixin,
    ModelMixin,
    RetrieveModelMixin,
    UpdateModelMixin,
)


class GenericViewSet(ViewSetMixin, GenericAPIView):
    pass


class ViewSet(MetadataMixin, ViewSet):
    def get_serializer(self):
        return self.serializer_class()

    def get_serializer_class(self):
        return getattr(self, "serializer_class", None)


class ReadOnlyModelViewSet(
    MetadataMixin,
    ModelMixin,
    RetrieveModelMixin,
    ListModelMixin,
    FilterMixin,
    GenericViewSet,
):
    pagination_class = CursorPagination
    READ_ONLY = True
    CREATE_BUTTONS = []
    BUTTONS = [Button.REFRESH.value]


class ModelViewSet(
    FSMViewSetMixin,
    MetadataMixin,
    ModelMixin,
    CreateModelMixin,
    RetrieveModelMixin,
    UpdateModelMixin,
    DestroyModelMixin,
    DestroyMultipleModelMixin,
    ListModelMixin,
    FilterMixin,
    GenericViewSet,
):
    pagination_class = CursorPagination


class ReadOnlyInfiniteModelViewSet(ReadOnlyModelViewSet):
    pagination_class = None


RepresentationModelViewSet = ReadOnlyModelViewSet


class InfiniteDataModelView(ModelViewSet):
    pagination_class = None


class ChartViewSet(FilterMixin, ViewSet):

    WIDGET_TYPE = WidgetType.CHART.value

    def list(self, request: Request, *args, **kwargs):
        figure = self.get_plotly(self.filter_queryset(self.get_queryset()))
        figure_dict = figure.to_plotly_json()
        figure_dict["config"] = {"responsive": True, "displaylogo": False}
        figure_dict["useResizeHandler"] = True
        figure_dict["style"] = {"width": "100%", "height": "100%"}
        return Response(figure_dict)

    def get_queryset(self):
        return self.queryset

    def filter_queryset(self, queryset: QuerySet) -> QuerySet:
        for backend in list(self.filter_backends):
            queryset = backend().filter_queryset(self.request, queryset, self)
        return queryset



from typing import Dict, Union

from rest_framework.request import Request

from bridger.display import Calendar, InstanceDisplay, ListDisplay
from bridger.metadata.mixins import BridgerMetadataMixin


class InstanceDisplayMetadata(BridgerMetadataMixin):
    key = "instance_display"
    method_name = "_get_instance_display"


class InstanceDisplayMetadataMixin:
    def get_instance_display(
        self, request: Request, display: InstanceDisplay = None
    ) -> InstanceDisplay:
        return display or []

    def _get_instance_display(self, request: Request) -> Dict:
        return list(
            self.get_instance_display(
                request=request, display=getattr(self, "INSTANCE_DISPLAY", None)
            )
        )


class ListDisplayMetadata(BridgerMetadataMixin):
    key = "list_display"
    method_name = "_get_list_display"


class ListDisplayMetadataMixin:
    def get_list_display(
        self, request: Request, display: Union[ListDisplay, Calendar] = None
    ) -> Union[ListDisplay, Calendar]:
        return display or {}

    def _get_list_display(self, request: Request) -> Dict:
        return dict(
            self.get_list_display(
                request=request, display=getattr(self, "LIST_DISPLAY", None)
            )
        )



from rest_framework.metadata import BaseMetadata

from bridger.enums import WidgetType

from bridger.metadata.fields import (
    ButtonMetadata,
    CreateButtonMetadata,
    CustomButtonMetadata,
    CustomInstanceButtonMetadata,
    EndpointMetadata,
    FieldsMetadata,
    FilterFieldsMetadata,
    IdentifierMetadata,
    InstanceDisplayMetadata,
    ListDisplayMetadata,
    OrderingFieldsMetadata,
    PaginationMetadata,
    PKMetadata,
    SearchFieldsMetadata,
    TitleMetadata,
    WidgetTypeMetadata,
)


# class PandasMetadata(BaseMetadata):
#     def determine_metadata(self, request, view):
#         metadata = defaultdict(dict)
        
#         metadata["identifier"] = "pandas:1234"
#         metadata["fields"] = view.pandas_fields.to_dict()
#         metadata["list_display"] = view.get_list_display(request)
#         metadata["buttons"] = ["refresh"]

#         # metadata["search_fields"] = view._get_search_fields(request)
#         # metadata["ordering_fields"] = view._get_ordering_fields(request)
#         # metadata["filter_fields"] = {k: v for k, v in view._get_filter_fields(request)}

#         metadata["create_buttons"] = []
#         metadata["custom_buttons"] = []
#         metadata["custom_instance_buttons"] = []

#         # metadata["endpoints"] = view.get_endpoints(
#         #     request=request, buttons=metadata["buttons"]
#         # )
#         # metadata["titles"] = view.get_titles(request=request)
#         return metadata

class PandasMetadata(BaseMetadata):
    def __init__(self, *args, **kwargs):
        self.modules = [
            # IdentifierMetadata,
            # WidgetTypeMetadata,
            TitleMetadata,
            EndpointMetadata,
            ButtonMetadata,
            # CreateButtonMetadata,
            CustomButtonMetadata,
            CustomInstanceButtonMetadata,
            SearchFieldsMetadata,
            OrderingFieldsMetadata,
            FilterFieldsMetadata,
            ListDisplayMetadata,
        ]
        super().__init__(*args, **kwargs)

    def generate_metadata(self, request, view):
        for module in self.modules:
            yield from module(view, request)

    def generate_model_metadata(self, request, view):
        for module in self.model_modules:
            yield from module(view, request)

    def determine_metadata(self, request, view):
        metadata = dict(self.generate_metadata(request, view))

        metadata["type"] = "list"
        metadata["pagination"] = None
        metadata["identifier"] = "pandas:1234"
        metadata["fields"] = view.pandas_fields.to_dict()
        # metadata["buttons"] = ["refresh"]

        return metadata



from rest_framework import filters
from rest_framework.request import Request

from bridger.metadata.mixins import BridgerMetadataMixin


class SearchFieldsMetadata(BridgerMetadataMixin):
    key = "search_fields"
    method_name = "_get_search_fields"


class SearchFieldsMetadataMixin:
    def _get_search_fields(self, request: Request):
        if filters.SearchFilter in getattr(self, "filter_backends", []):
            yield from self.search_fields



from typing import List

from rest_framework.request import Request

from bridger.enums import Button
from bridger.metadata.mixins import BridgerMetadataMixin


class CreateButtonMetadata(BridgerMetadataMixin):
    key = "create_buttons"
    method_name = "_get_create_buttons"


class CreateButtonMetadataMixin:
    def get_create_buttons(
        self, request: Request, buttons: List[str] = None
    ) -> List[str]:
        return buttons or Button.create_buttons()

    def _get_create_buttons(self, request: Request) -> List[str]:
        return self.get_create_buttons(
            request=request, buttons=getattr(self, "CREATE_BUTTONS", None)
        )
